from izzylib import DotDict, logging
from izzylib.cache import TtlCache
from izzylib.http_server import UserLevel, View

from ..config import config
from ..database import db
from ..functions import Captcha


cache = DotDict(
	captcha = TtlCache(ttl='30m')
)


class Home(View):
	paths = ['/']

	async def get(self, request, response):
		return response.template('page/home.haml')


class Register(View):
	paths = ['/register']

	async def get(self, request, response, data={}, msg=None, error=None):
		capt = Captcha()

		if logging.get_config('level') == logging.Levels.DEBUG:
			capt.linecolor = (0,0,0,0)

		captcha_base64 = capt.get_captcha().decode()
		cache.captcha.store(captcha_base64, capt)

		data = {
			'captcha_base64': captcha_base64,
			'form': data
		}

		if msg:
			data['message'] = msg

		if error:
			data['error'] = error

		return response.template('page/register.haml', data)


	async def post(self, request, response):
		capt_img = request.data.form.pop('captcha_base64', None)
		capt_text = request.data.form.pop('captcha_text', '')
		capt = cache.captcha.pop(capt_img, None)

		handle = request.data.form.get('handle')
		password = request.data.form.get('password')
		password2 = request.data.form.get('password2')
		display = request.data.form.get('display', handle)

		if not handle:
			return await self.get(request, response, data=request.data.form, error='Missing handle')

		if not password or not password2:
			return await self.get(request, response, data=request.data.form, error='Missing one or both passwords')

		if password != password2:
			return await self.get(request, response, data=request.data.form, error='Passwords do not match')

		if not capt_img:
			return await self.get(request, response, data=request.data.form, error='Missing captcha')

		if not capt:
			return await self.get(request, response, data=request.data.form, error='Invalid captcha')

		if capt_text.lower() != capt.text.lower():
			return await self.get(request, response, data=request.data.form, error='Captcha text does not match')

		user = db.put('user', handle, password, display)
		token = db.put('token', user.id)
		token.set_cookie(response)

		return response.redir('/account/home')


class Login(View):
	paths = ['/login']

	async def get(self, request, response, msg=None, error=None, handle=None):
		data = {'handle': handle}

		if msg:
			data['message'] = msg

		if error:
			data['error'] = error

		return response.template('page/login.haml', context=data)


	async def post(self, request, response):
		username = request.data.form.get('handle')
		password = request.data.form.get('password')

		if not username:
			return await self.get(request, response, error='Missing username')

		if not password:
			return await self.get(request, response, error='Missing password', handle=username)

		try:
			if db.get('password', username, password):
				return response.redir('/account/home')

		except AttributeError:
			pass

		return await self.get(request, response, error='Invalid username or password', handle=username)


class Logout(View):
	paths = ['/logout']

	async def get(self, request, response):
		print(request.ctx.token)
		response = await Login.get(self, request, response, msg='Logged out')
		print(request.ctx.token)
		request.ctx.token.del_cookie(response)
		return response


class Account(View):
	paths = ['/account', '/account/<page>']

	async def get(self, request, response, page=None):
		if not page:
			return response.redir('/account/home')

		return response.template(f'page/account/{page}.haml')


	async def post(self, request, response):
		pass


class Admin(View):
	paths = ['/admin', '/admin/<page>']

	async def get(self, request, response):
		if not page:
			return response.redir('/admin/home')

		return response.template(f'page/admin/{page}.haml')


	async def post(self, request, response):
		pass
